/*
 *  Spot and Texture OpenGL Demonstrator
 *  Moving parallel or spot light with textures and ambient light control
 *
 *  Created by brisson on 2012/05/02.
 *  Updated by brisson on 2013/05/04.
 *  Copyright (c) 2012-2013 J M. Brisson Lopes & Instituto Superior Tecnico. All rights reserved.
 *
 */

#pragma once

#include <string>
#include <vector>
using std::string;
using std::vector;

// Object Manager - controls tiles, textures and a sphere

class ObjectManager {
public:
    ObjectManager();
    ~ObjectManager();
    void Draw() const;
    const bool ShowSphere();
    const bool SelectTexture( const int textureNo);
    void       ChangeNoOfDivisions( const bool increase);
    const int  NoOfDivisions() const { return Divisions[ DivisionsIndex]; }
    const bool ShowingTexture() const;
protected:
    vector<unsigned int> Textures;
    unsigned int Sphere, Board;
    bool         DrawSphere;
    int          NoTextureToUse, DivisionsIndex;
    
    const unsigned int LoadTexture(   const string & textureFilename);
    const unsigned int GenerateBoard( const int noDivisions);
    
    static float DiffuseTextureOff[], DiffuseTextureOn[];
    static int   Divisions[];
};


// Camera - does not do much once set


class Camera {
public:
    Camera();
    void SetProjection();
protected:
};


// Lightsource - controls light source type (spor or direccional) and global ambient


class LightSource {
public:
    LightSource();
    void       Locate() const;
    const bool SetIncidenceAngle( const bool increase);
    const int  LightSourceAngle() const { return LightAngle; }
    const bool ToggleLightSource();
    const bool ToggleAmbientLight();
    const bool IsSpotOn() const { return SpotlightOn; }
    const bool IsAmbientOn() const { return AmbientOn; }
protected:
    bool  SpotlightOn;
    bool  AmbientOn;
    int   LightAngle;
    static float LightPosition[], LightDirection[], AmbientModel[], NoAmbientModel[];
    static int   AngleIncrement, MaxAngle, MinAngle;
};


// The application


class OpenGLApplication {
    
public:
    OpenGLApplication();
    ~OpenGLApplication();
    void Initialize( int argc, char ** argv);
    void MyDisplay();
    void MyReshape( const int w, const int h);
    void MyKeyboard( const unsigned char key, const int x, const int y);
    void MySpecialFunction( const int key, const int x, const int y);
    
    static OpenGLApplication MyApplication;
protected:
    LightSource   * MyLightSource;
    Camera        * MyCamera;
    ObjectManager * MyObjects;
    bool            MyTimerOn;
    bool            FullScreenOn;
    bool            HUDon;
    void            HUD();
    void            DisplayString( const int x, const int y, const string & label,
                                   const void * font = NULL);
    static void   display() { MyApplication.MyDisplay(); }
    static void   reshape( int w, int h) { MyApplication.MyReshape( w, h); }
    static void   keyboard( unsigned char key, int x, int y) { MyApplication.MyKeyboard( key, x, y); }
    static void   special( int key, int x, int y) { MyApplication.MySpecialFunction( key, x, y); }
    static string label1, label2, label3;
};
